#!c:/perl/bin/perl -w
#!/usr/bin/perl

# use strict;

# Set to 1 for debugging locally, 0 before uploading to internet.
# If you don't know what this means, then set this to 0 and only set
# the variables below for the line after '} else {'.

my $local="1";


# Declare them variables.

my ($path, $url, $headlinepage, $template, $countfile, $script,
$cgilib, $newspagedir, $stylesheet, $flock, $stylesheetwebaddress);


#####################################################################
#
#  News Editor. Copyright (C) Brendan Arnold 2002. For use on the
#  FRANK MORGAN SCHOOL OF FLYING site http://www.flyatfranks.co.uk/
#
#####################################################################
#
# Overview: This script provides a webform interface to write, edit
# and delete news items each on their own dedicated .html news page.
# It also inserts linked headlines on an index page.
#
# NOTE: This script does not have any password protection built in.
# Before you place it online, make sure that you protect it with
# .htaccess or similar means.
#
# This is the first working version of the script and so is likely to
# be buggy. Soon I hope to convert the script to 'use strict' as well
# as incorperate more features. Below is a list of such feaures which
# I hope to soon include.
#
# * Conversion of form input into valid xhtml.
# * Conversion of xhtml into more readable form input.
#
# Disclaimer: If you choose to use this script then you agree that no
# responsibility can be pinned on the author in the event of data
# loss or harm to a system. No support will be given by the author.
#
#####################################################################
#
#                          SET VARIABLES
#
# These are variables which you will have to configure for the script
# to work on your system. All values should be contained inside double
# quotes and the line should terminate with a semi-colon. Further 
# details are given with each variable.
#

# Insert your full server path inbetween the quotes including 
# trailing slash.

if ($local) {

$path="/your/local/path/to/root/directory";

} else {

$path="/your/server/path/to/root/directory";

}

# Insert your base web URL inbetween the quote including trailing 
# slash and 'htpp://'.

if ($local) {

$url="http://localhost/";

} else {

$url="http://www.flyatfranks.co.uk/";

}


# The rest of the files are locations relative to the root with no 
# beginning or final slash. So if your file was located on the web at
# www.example.com/news/index.html, then you would put news/index.html
# inbetween the quotes.

# The page where you want the headlines inserted. Must include 
# <!--NEWS_HEADLINES_START--> and <!--NEWS_HEADLINES_END--> in the 
# file itself inbetween which you want the headlines inserted.

if ($local) {

$headlinepage="htdocs/headlinepage.html";

} else {

$headlinepage="headlinepage.html";

}


# The html template code for each news page. Must include 
# <!--INSERT_NEWS_TITLE_HERE--> where you want the title to appear and 
# <!--INSERT_NEWS_BODY_HERE--> where you want the main text to appear.

if ($local) {

$template="cgi-bin/news/article_template.txt";

} else {

$template="cgi-bin/news/article_template.txt";

}

# Location and name of the count file relative to the root with no 
# beginning slash.

if ($local) {

$countfile="cgi-bin/news/newscount.txt";

} else {

$countfile="cgi-bin/news/newscount.txt";

}

# Location and name of this script relative to the root with no 
# beginning slash.

if ($local) {

$script="cgi-bin/news/news.cgi";

} else {

$script="cgi-bin/news/news.cgi";

}

# Location and name of cgi-lib.pl relative to the root with no 
# beginning slash

$cgilib="cgi-bin/cgi-lib.pl";


# Location of the directory where the news pages will be stored 
# relative to the root. No trailing slash or starting slash needed.

if ($local) {

$newspagedir="htdocs/news/";

} else {

$newspagedir="news/";

}

# If you want to use a stylesheet to alter the look of the editor's 
# pages then specify its location relative to the root here. 
# Otherwise leave nothing between the quotes.

if ($local) {

$stylesheet="cgi-bin/news/scriptstyle.css";

} else {

$stylesheet="cgi-bin/news/scriptstyle.css";

}

# Set to 0 if your server does not support file locking. If in doubt, 
# set it to 0 until you get the script working then try setting it to
# 1.

if ($local) {

$flock="0";

} else {

$flock="1";

}

# STOP RIGHT THERE!! 
#
# End of user defined variables. Not necessary to edit beyond this 
# point. Html output is all nestled between HTML_CHUNK tags and so 
# can be edited with minimum fuss so long as you remember to comment
# out &, $ and % with a \. So, for example, &amp; becomes \&amp;.
#
#####################################################################


# Form filepaths.

my $headlinepagepath=$path.$headlinepage;
my $templatepath=$path.$template;
my $countfilepath=$path.$countfile;
my $cgilibpath=$path.$cgilib;
my $newspagedirpath=$path.$newspagedir;


# Form URLs.

my $scriptwebaddress=$url.$script;

if ($stylesheet) {

	$stylesheetwebaddress=$url.$stylesheet;

}


# Use cgi-lib.pl to parse data.

require "$cgilibpath";

&ReadParse(*form_data);


#####################################################################

# Start of the general procedure.


# If no form data supplied then gives the options.

unless (%form_data) {

	&header;
	&options;
	&footer;
	exit;

} elsif ($ENV{'HTTP_REFERER'} ne $scriptwebaddress) {

# Use HTTP_REFERER to check if script if self reffered. If not, 
# generates error.

	&CgiDie("This script is only supposed to function when accessed internally.","See help file for details.");

} elsif ($form_data{listnewsforedit}) {

	&header;
	&listnewsforedit;
	&footer;
	exit;

} elsif ($form_data{writenewnews}) {

	&header;
	&writenewnews;
	&footer;
	exit;

} elsif ($form_data{listnewsfordelete}) {

	&header;
	&listnewsfordelete;
	&footer;
	exit;

} elsif ($form_data{edit}=~/^editnews(\d+)?/) {

	&header;
	&editnews($1);
	&footer;
	exit;

} else {

	&deletetest;
	&writepagetest;

	# If still nothing satisfied, then main options displayed.

	&header;
	&options;
	&footer;
	exit;

}

# Begin sub routines


#####################################################################
#                            &header;                               #
#####################################################################

# Prints a html header.

sub header {

	# Tells browser that output is html.

	print "Content-type: text/html\n\n";

	print <<HTML_CHUNK;
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "DTD/xhtml1-transitional.dtd">

<html>
<head>

<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
<meta name="robots" content="none" />

<title>News Editor</title>

HTML_CHUNK

	# Includes stylesheet reference if specified

	if ($stylesheetwebaddress) {

		print <<HTML_CHUNK;

<link rel="stylesheet" type="text/css" href="$stylesheetwebaddress" />

HTML_CHUNK

	}

	print <<HTML_CHUNK;

</head>
<body>

HTML_CHUNK

}

#####################################################################
#                            &footer;                               #
#####################################################################

# Print a html footer.

sub footer {

	print <<HTML_CHUNK;

<hr size="1" style="margin-top:2%" />

<address style="font-style:italic"><small>Script courtesy of <a href="mailto:brendanarnold\@hotmail.com" title="Email the author with feedback at &lt;brendanarnold\@hotmail.com&gt;">B. Arnold</a>. Copyright &copy; 2002.</small></address>

</body>
</html>

HTML_CHUNK

}

#####################################################################
#                            &options;                              #
#####################################################################

# Prints options available in html.

sub options {

	my $confirmtitle="@_";

# Prints up to form header

	print <<HTML_CHUNK;
<h1>News Editor - Options</h1>

<hr size="1" />

HTML_CHUNK

	if ($confirmtitle) {

		print <<HTML_CHUNK;

<h2 class="confirmtitle">$confirmtitle</h2>

HTML_CHUNK

	}

	print <<HTML_CHUNK;

<h2 align="center">What do you want to do?</h2>

<table align="center" border="0" cellpadding="3">

<tr><td>
<form action="$scriptwebaddress" method="post">
<input type="hidden" value="writenewnews" name="writenewnews" />
<input type="submit" value="Write a New Article" />
</form>
</td>

<td>
<form action="$scriptwebaddress" method="post">
<input type="hidden" value="listnewsforedit" name="listnewsforedit" />
<input type="submit" value="Edit an Article" />
</form>
</td>

<td>
<form action="$scriptwebaddress" method="post">
<input type="hidden" value="listnewsfordelete" name="listnewsfordelete" />
<input type="submit" value="Delete an Article" />
</form>
</td></tr>

</table>

HTML_CHUNK

}



#####################################################################
#                         &listnewforedit;                          #
#####################################################################

# Prints html form to select a news article for editing.

sub listnewsforedit {

print <<HTML_CHUNK;

<h1>News Editor - Articles to Edit</h1>

<hr size="1" />

<h2 align="center">Which article do you want to edit?</h2>

<p><small>Note: clicking on the titles allows you to view the article in a separate window.</small></p>

<form action="$scriptwebaddress" method="post">

<table align="center" border="0" cellpadding="3">

HTML_CHUNK

	opendir DIR, "$newspagedirpath" or &CgiDie("Cannot open directory $newspagedirpath","$!");

	while ($file=readdir DIR) {

		if ($file=~/^(\d+)\.html?/) {

			$file=$newspagedirpath.$file;

			if ($local) {

				$pagewebaddress=$url."news/".$1.".html";

			} else {

				$pagewebaddress=$url.$newspagedir.$1.".html";

			}

			my $file=$newspagedirpath.$1.".html";

			my $title=&get_title($file);

			$title=&parsehtml($title);

			print <<HTML_CHUNK;

<tr><td class="button_cell">Article no. $1:</td><td class="main_cell"><big><a href="$pagewebaddress" target="_blank" title="View &quot;$title&quot; in new window"><cite>$title</cite></a></big></td><td class="button_cell"><input type="radio" value="editnews$1" name="edit" /></td></tr>

HTML_CHUNK

		}

	}

	print <<HTML_CHUNK;
</table>

<p align="center"><input type="submit" value="Edit Selected Article" /></p>

</form>

<hr size="1" />

<form action="$scriptwebaddress" method="post"><p><input type="submit" value="Return to Main Options" /></p></form>

HTML_CHUNK

}

#####################################################################
#                       &listnewsfordelete;                         #
#####################################################################

# Prints html list of articles for deleting.

sub listnewsfordelete {

print <<HTML_CHUNK;

<h1>News Editor - Articles to Delete</h1>

<hr size="1" />



<h2 align="center">Which article(s) do you want to delete?</h2>

<p><small>Note: clicking on the titles allows you to view the article in a separate window.</small></p>

<form action="$scriptwebaddress" method="post">

<table align="center" border="0" cellpadding="3">


HTML_CHUNK

	opendir DIR, "$newspagedirpath" or &CgiDie("Cannot open directory $newspagedirpath: $!");

	while ($file=readdir DIR) {

		if ($file=~/^(\d+)\.html?/) {

			$file=$newspagedirpath.$file;

			if ($local) {

			$pagewebaddress=$url."news/".$1.".html";


			} else {

			$pagewebaddress=$url.$newspagedir.$1.".html";

			}

			my $file=$newspagedirpath.$1.".html";

			my $title=&get_title($file);

			$title=&parsehtml($title);

			print <<HTML_CHUNK;

<tr><td class="number_cell">Article no. $1:</td><td class="main_cell"><big><a href="$pagewebaddress" target="_blank" title="View &quot;$title&quot; in new window"><cite>$title</cite></a></big></td><td class="button_cell"><input type="checkbox" value="deletenews$1" name="deletenews$1" /></td></tr>

HTML_CHUNK

		}

	}

	print <<HTML_CHUNK;

</table>

<p align="center"><span style="color:red;font-weight:bold"><small>WARNING: Once you push this button selected articles cannot be retreived</small></span><br /><input type="submit" value="Delete Selected Article(s)" /></p>

</form>

<hr size="1" />

<form action="$scriptwebaddress" method="post"><p><input type="submit" value="Return to Main Options" /></p></form>

HTML_CHUNK

}

#####################################################################
#                           &get_title;                             #
#####################################################################

# Parses a file given in parameter for whatever lies within custom
# tags and returns it.

sub get_title {

	my $file="@_";

	local undef $/;

	open FILE, "$file" or &CgiDie("Cannot open $file for read");

	if ($flock) {

		flock(FILE, 1);

	}

	while (<FILE>) {

		if (/<!--TITLE_START-->(.*?)<!--TITLE_END-->/sm ) { # /s makes . cross line boundaries

			$title=$1;

		} 

# Error capture incase custom tags not included in news file.

		else {

			$title="[Could not get title. See help file for advice.]";

		}

	}

	close FILE;

	return($title);

}

#####################################################################
#                          &get_body;                               #
#####################################################################

# Parses file given in parameter for whatever lies in custom tags and
# returns it.

sub get_body {

	my $file="@_";

	local undef $/;

	open FILE, "$file" or &CgiDie("Cannot open $file for read");

	if ($flock) {

		flock(FILE, 1);

	}

	while (<FILE>) {

		if (/<!--BODY_START-->(.*?)<!--BODY_END-->/sm) {

			$body=$1;

		}

# Error capture incase custom tags not included in news file.

		else {

			$body="[Could not get main body. See help file for advice.]";

		}

	}


	close FILE;

	return($body);

}

#####################################################################
#                        &writenewnews;                             #
#####################################################################

# Prints html form for writing a new article and submitting it.

sub writenewnews {

	open FILE, "$countfilepath" or &CgiDie("Cannot open $countfilepath for read");

	if ($flock) {

		flock(FILE, 1);

	}

	$count=<FILE>;

	close FILE;

	print <<HTML_CHUNK;

<h1>News Editor - Writing New Article (No. $count)</h1>

<hr size="1" />

<h2 align="center">Write your article below then push <cite>Add This Article</cite>.</h2>

<form action="$scriptwebaddress" method="post">

<table align="center" border="0">

<tr><td><input type="text" name="title$count" value="&lt;h1&gt;Write title here (requires html tags)&lt;/h1&gt;" size="75" /></td></tr>

<tr><td><textarea rows="20" cols="75" name="body$count">&lt;p&gt;Write main text here (Requires html tags.)&lt;/p&gt;</textarea></td></tr>

</table>

<p align="center"><input type="hidden" name="updatecount" value="updatecount" /><input type="submit" value="Add This Article" /></p>

</form>

<hr size="1" />

<form action="$scriptwebaddress" method="post"><p><input type="submit" value="Return to Main Options" /></p></form>

HTML_CHUNK

}

#####################################################################
#                            &editnews;                             #
#####################################################################

# Prints update form along with contents of file whose index number 
# is given in parameter.

sub editnews {

	my $count="@_";

	my $file=$newspagedirpath.$count.".html";

	my $title=&get_title($file);

	my $body=&get_body($file);

	$title=&subentity($title);

	$body=&subentity($body);

	print <<HTML_CHUNK;

<h1>News Editor - Editing Article No. $count</h1>

<hr size="1" />

<h2 align="center">Make your changes below then push <cite>Update News</cite></h2>

<form action="$scriptwebaddress" method="post">

<table align="center" border="0">

<tr><td><input type="text" name="title$count" value="$title" size="75" /></td></tr>

<tr><td><textarea rows="20" cols="75" name="body$count">$body</textarea></td></tr>

</table>

<p align="center"><input type="submit" value="Update News" /></p>

</form>

<hr size="1" />

<form action="$scriptwebaddress" method="post"><p><input type="submit" value="Return to Main Options" /></p></form>

HTML_CHUNK

}

#####################################################################
#                          &createpage;                             #
#####################################################################

# Reads template file and creates/rewrites file given in parameter. 
# Also updates the countfile if necessary.

sub createpage {

	my $matchok;

	my ($file,$count)=@_;

	my $title='title'.$count;

	my $body='body'.$count;

	local undef $/;

	open TEMPLATE, "$templatepath" or &CgiDie("Cannot open $templatepath for read: $!");

	if ($flock) {

		flock(TEMPLATE, 1);

	}

	while (<TEMPLATE>) {

		if (/(.*)<!--INSERT_NEWS_TITLE_HERE-->(.*)<!--INSERT_NEWS_BODY_HERE-->(.*)/sm) {

			$header=$1;

			$spacer=$2;

			$footer=$3;

		}

# Error capture incase custom tags not found in template file.

		else {

			$header="<html><head><title>[Check Template File]</title></head><body>";
			$spacer="";
			$footer="</body></html>";

			$matchok=1;

		}

	}

	close TEMPLATE;

	open FILE, ">$file" or &CgiDie("Cannot open $file for write: $!");

	if ($flock) {

		flock(FILE, 2);

	}

	print FILE $header;

	print FILE <<HTML_CHUNK;

<!--TITLE_START-->$form_data{$title}<!--TITLE_END-->

HTML_CHUNK

	print FILE $spacer;

	print FILE <<HTML_CHUNK;

<!--BODY_START-->$form_data{$body}<!--BODY_END-->

HTML_CHUNK

	print FILE $footer;

	close FILE;

	&writeheadlines;

	if ($form_data{'updatecount'}) {

		open FILE, "$countfilepath" or &CgiDie("Cannot open $countfilepath for read: $!");

		if ($flock) {

			flock(FILE, 1);

		}

		$count=<FILE>;

		if ($count<=999) {

			$count++;

		} else {

			$count=1;

		}

		close FILE;

		open FILE, ">$countfilepath" or &CgiDie("Cannot open $countfilepath for write: $!");

		if ($flock) {

			flock(FILE, 2);

		}

		print FILE $count;
	
		close FILE;

	}

	return($matchok);

}


#####################################################################
#                          &writeheadlines;                         #
#####################################################################

# Opens and rewrites headlines file including html links to all news 
# files in newspage directory.

sub writeheadlines {

	my $matchok;

	local undef $/;

	open FILE, "$headlinepagepath" or &CgiDie("Cannot open $headlinepagepath for read: $!");

	if ($flock) {

		flock(FILE, 1);

	}

	while (<FILE>) {

		if (/(.*)<!--NEWS_HEADLINES_START-->.*<!--NEWS_HEADLINES_END-->(.*)/sm) {

			$header=$1;

			$footer=$2;

		} 

# Error capture incase custom tags not found in headline file.

		else {

			$matchok=2;

		}

	}

	close FILE;

	unless ($matchok == 2) {

		opendir DIR, "$newspagedirpath" or &CgiDie("Cannot open directory $newspagedirpath: $!");

		while ($file=readdir DIR) {

			if ($file=~/^(\d+)\.html?/) {


				if ($local) {
				
					$filewebaddress=$url.'news/'.$file;

				} else {

					$filewebaddress=$url.$newspagedir.$file;

				}

				my $file=$newspagedirpath.$file;

				$newsaddress{$filewebaddress}=&get_title($file);

			}

		}


		open FILE, ">$headlinepagepath" or &CgiDie("Cannot open $headlinepagepath for write: $!");

		if ($flock) {

			flock(FILE, 2);

		}

		print FILE $header;

		print FILE <<HTML_CHUNK;
<!--NEWS_HEADLINES_START-->
<ul style="margin-top:1em">

HTML_CHUNK

		foreach (keys %newsaddress) {

			print FILE <<HTML_CHUNK;

<li><a href="$_">$newsaddress{$_}</a></li>

HTML_CHUNK

		}

	print FILE <<HTML_CHUNK;

</ul>
<!--NEWS_HEADLINES_END-->
HTML_CHUNK

		print FILE $footer;

		close FILE;

	}

}

#####################################################################
#                            &subentity;                            #
#####################################################################

# Simple substitution of entities to ensure valid html in forms. Note
# this does NOT validate user output.

sub subentity {

	my $input="@_";

	
	# Add to this hash any entities you want converting from html to 
	# form input when displayed on edit page.

	my %entities=(

		'&lt;',	'<',
		'&gt;',	'>'

	);

	foreach (keys %entities) {

		$input=~s/$entities{$_}/$_/g;

	}

	return($input);

}

#####################################################################
#                           &deletetest;                            #
#####################################################################

# Carries out deletions of files if certain form_data values are 
# true.

sub deletetest {

	my $update=0; # For deletion confirmation later on.

	foreach (keys %form_data) {

		if (/^deletenews(\d+)?/) {

			my $file=$newspagedirpath.$1.'.html';

			unlink ($file) or &CgiDie("Cannot delete $file: $!");

			delete $form_data{$_}; # Then need to remove the element from the hash

			$update++; # Is true if a deletion has occured.

		} 

	}

	if ($update) { # Updates if a deletion took place

		my $confirmtitle;

		&writeheadlines;

		if ($update == 1) {

		$confirmtitle="1 News article successfully removed";

		} else {

		$confirmtitle="$update News articles successfully removed";

		}

		&header;
		&options($confirmtitle);
		&footer;
		exit;

	}

}

#####################################################################
#                          &writepagetest;                          #
#####################################################################

# Checks form_data to see if a page needs to be written.

sub writepagetest {


# Tests to see if $form_data{title[number]} exists and if does then
# writes new page.

	foreach (keys %form_data) {

		if (/^title(\d+)?/) {

			my $confirmtitle;

			my $file=$newspagedirpath.$1.'.html';

			my $matchok=&createpage($file,$1);

			unless ($matchok) {

				$confirmtitle="News page successfully written";

			} elsif ($matchok == 1) {

				$confirmtitle="Page written but template file does not include necessary tags. See the help guide for how to fix this."; 

			} elsif ($matchok == 2) {

				$confirmtitle="Headlines page not updated because necssary tags weren't found. See help guide for how to fix this.";

			}

			&header;
			&options($confirmtitle);
			&footer;
			exit;

		}

	}

}

#####################################################################
#                           &parsehtml;                             #
#####################################################################

# Removes all html tags from a string given in parameter.

sub parsehtml {

	my $text="@_";

	$text=~s/<.+?>//g;

	return($text);

}
